{*********************************************************}
{*          Web server application components            *}
{*                     version 2.0                       *}
{*         Copyright (c) 2000-2002  S.Perevoznyk         *}
{*                 All rights reserved.                  *}
{*********************************************************}
Unit psvDBWeb;

interface

uses
  Classes, HttpAPP, psvWeb, DB, Jpeg, {$IFDEF VER140}HTTPProd,{$ENDIF} Graphics;

type

 TpsvDataSetPageProducer = class(TpsvPageProducer)
  private
    FDataSet: TDataSet;
  protected
    function GetDataSet: TDataSet; virtual;
    procedure SetDataSet(ADataSet: TDataSet); virtual;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure DoTagEvent(Tag: TTag; const TagString: string; TagParams: TStrings;
      var ReplaceText: string); override;
  public
    function Content: string; override;
  published
    property DataSet: TDataSet read GetDataSet write SetDataSet;
    property OnHTMLTag;
  end;


  TpsvDBCtrlProducer = class(TpsvDataSetPageProducer)
  private
   FMaxRows : integer;
  public
    function Content : string; override;
  published
    property MaxRows : integer read FMaxRows write FMaxRows;
  end;

  TpsvDBPageProducer = class(TpsvPageProducer)
  private
    FDataSet: TDataSet;
    FIndexName : string;
    FFieldName : string;
    FIndex     : string;
  protected
    function  GetDataSet: TDataSet; virtual;
    procedure SetDataSet(ADataSet: TDataSet); virtual;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    function    Content: string; override;
    procedure   AppendPage(Value : TStringList);
    procedure   ChangePage(Value : TStringList);
  published
    property DataSet: TDataSet read GetDataSet write SetDataSet;
    property Index : string read FIndex write FIndex;
    property IndexName : string read FIndexName write FIndexName;
    property FieldName : string read FFieldName write FFieldName;
  end;


 TpsvDBContentProducer = class(TComponent)
   private
     FDataSet: TDataSet;
     FFieldName : string;
     FIndexName : string;
     FIndex     : String;
     FTypeField : string;
   protected
    function  GetDataSet: TDataSet; virtual;
    procedure SetDataSet(ADataSet: TDataSet); virtual;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    function  GetContentAsStream: TStream; virtual;
    function  GetContentType: string; virtual;
   public
    property ContentType    : string  read GetContentType;
    property ContentAsStream: TStream read GetContentAsStream;
   published
    property DataSet: TDataSet read GetDataSet write SetDataSet;
    property FieldName : string read FFieldName write FFieldName;
    property IndexName : string read FIndexName write FIndexName;
    property TypeField : string read FTypeField write FTypeField;
    property Index : String read FIndex write FIndex;
  end;

procedure Register;

implementation

{$R NotFound.res}

Procedure Register;
begin
  RegisterComponents('FastWeb',[TpsvDataSetPageProducer, TpsvDBPageProducer,
                                 TpsvDBCtrlProducer, TpsvDBContentProducer]);
end;

function TpsvDataSetPageProducer.GetDataSet: TDataSet;
begin
  Result := FDataSet;
end;

procedure TpsvDataSetPageProducer.SetDataSet(ADataSet: TDataSet);
begin
  if FDataSet <> ADataSet then
  begin
    if ADataSet <> nil then ADataSet.FreeNotification(Self);
    FDataSet := ADataSet;
  end;
end;

procedure TpsvDataSetPageProducer.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (AComponent = FDataSet) then
    FDataSet := nil;
end;

procedure TpsvDataSetPageProducer.DoTagEvent(Tag: TTag; const TagString: string;
  TagParams: TStrings; var ReplaceText: string);
var
  Field: TField;
begin
  if (TagParams.Count = 0) and Assigned(FDataSet) then
  begin
    Field := FDataSet.FindField(TagString);
    if Assigned(Field) then
        ReplaceText := Field.DisplayText;
  end;
  inherited DoTagEvent(Tag, TagString, TagParams, ReplaceText);
end;

function TpsvDataSetPageProducer.Content: string;
begin
  if (FDataSet <> nil) and not FDataSet.Active then
    FDataSet.Open;
  Result := inherited Content;
end;


//TpsvDBPageProducer

function TpsvDBPageProducer.GetDataSet: TDataSet;
begin
  Result := FDataSet;
end;

procedure TpsvDBPageProducer.SetDataSet(ADataSet: TDataSet);
begin
  if FDataSet <> ADataSet then
  begin
    if ADataSet <> nil then ADataSet.FreeNotification(Self);
    FDataSet := ADataSet;
  end;
end;

procedure TpsvDBPageProducer.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (AComponent = FDataSet) then
    FDataSet := nil;
end;


function TpsvDBPageProducer.Content: string;
begin
  if (FDataSet <> nil) and not FDataSet.Active then
    FDataSet.Open;

  if FDataSet.Locate(FIndexName,FIndex,[]) then
   begin
     Result := ContentFromString(FDataSet.FieldByName(FFieldName).AsString);


   end
    else
      Result := inherited Content;
end;

procedure TpsvDBPageProducer.AppendPage(Value : TStringList);
begin
 try
  FDataSet.Append;
  FDataSet.FieldByName(FFieldName).AsString := Value.Text;
  FDataSet.Post;
 finally
 end;
end;


procedure TpsvDBPageProducer.ChangePage(Value : TStringList);
begin
 try
  FDataSet.Edit;
  FDataSet.FieldByName(FFieldName).AsString := Value.Text;
  FDataSet.Post;
 finally
 end;
end;



{ TpsvDBCtrlProducer }

function TpsvDBCtrlProducer.Content: string;
var I : integer;
    St : string;
begin
  St := '';
  if (DataSet <> nil) and not DataSet.Active then
    DataSet.Open;
  i := 0;
  DataSet.First;
  while not DataSet.Eof do
   begin
     inc(i);
     if ( (FMaxRows > 0) and (I > FMaxRows))  then
      break;
     St := St + inherited Content;
     DataSet.Next;
   end;
  Result := St;
end;


//TpsvDBContentProducer

function TpsvDBContentProducer.GetContentType : string;
var Field : TField;

begin
  if (FDataSet <> nil) and not FDataSet.Active then
    FDataSet.Open;

  if FDataSet.Locate(FIndexName,FIndex,[]) then

   begin
     Field := FDataSet.FindField(FTypeField);
      if Assigned(Field) then
       Result := Field.AsString;
   end
     else
       result := 'image/jpeg';
end;


function TpsvDBContentProducer.GetContentAsStream: TStream;
var
  DataField : TBlobField;

function ProduceNotFound: TStream;
var
  Jpg: TJpegImage;
  Bmp: TBitmap;
begin
  Result := TMemoryStream.Create;
    try
      Jpg    := TJpegImage.Create;
      Bmp := TBitmap.Create;
      Bmp.LoadFromResourceName(hInstance,'psvNOTFOUND');
      try
        Jpg.Assign(Bmp);
        Jpg.SaveToStream(Result);
        Result.Position := 0;
      finally
        Jpg.Free;
        Bmp.Free;
      end;
  except
    Result.Free;
    raise;
  end;
end;


begin

  if (FDataSet <> nil) and not FDataSet.Active then
    FDataSet.Open;

  if FDataSet.Locate(FIndexName,FIndex,[]) then
   begin
     DataField := TBlobField(FDataSet.FindField(FFieldName));
     if Assigned(DataField) then
       begin
         Result := TMemoryStream.Create;
         DataField.SaveToStream(Result);
         Result.Position := 0;
       end
         else
           Result := ProduceNotFound;
   end
    else
      Result := ProduceNotFound;
end;


function TpsvDBContentProducer.GetDataSet: TDataSet;
begin
  Result := FDataSet;
end;

procedure TpsvDBContentProducer.SetDataSet(ADataSet: TDataSet);
begin
  if FDataSet <> ADataSet then
  begin
    if ADataSet <> nil then ADataSet.FreeNotification(Self);
    FDataSet := ADataSet;
  end;
end;

procedure TpsvDBContentProducer.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (AComponent = FDataSet) then
    FDataSet := nil;
end;

end.